@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   ParaView
-  Module:    vtkCPGridBuilder.h
-
-  Copyright (c) Kitware, Inc.
-  All rights reserved.
-  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 /**
  * @class   vtkCPGridBuilder
  * @brief   Abstract class for creating grids.
@@ -41,13 +29,13 @@ public:
    */
   vtkDataObject* GetGrid(unsigned long timeStep, double time, int& builtNewGrid) override = 0;
 
-  //@{
+  ///@{
   /**
    * Set/get the FieldBuilder.
    */
   void SetFieldBuilder(vtkCPBaseFieldBuilder* fieldBuilder);
   vtkCPBaseFieldBuilder* GetFieldBuilder();
-  //@}
+  ///@}
 
 protected:
   vtkCPGridBuilder();
@@ -57,7 +45,7 @@ private:
   vtkCPGridBuilder(const vtkCPGridBuilder&) = delete;
 
   void operator=(const vtkCPGridBuilder&) = delete;
-  //@{
+  ///@{
   /**
    * The field builder for creating the input fields to the coprocessing
    * library.
