@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkUniformGrid.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "vtkUniformGrid.h"
 
@@ -33,6 +21,7 @@
 #include "vtkVertex.h"
 #include "vtkVoxel.h"
 
+VTK_ABI_NAMESPACE_BEGIN
 vtkStandardNewMacro(vtkUniformGrid);
 
 unsigned char vtkUniformGrid::MASKED_CELL_VALUE =
@@ -850,7 +839,7 @@ void vtkUniformGrid::BlankPoint(vtkIdType ptId)
 }
 
 //------------------------------------------------------------------------------
-void vtkUniformGrid::BlankPoint(const int i, const int j, const int k)
+void vtkUniformGrid::BlankPoint(int i, int j, int k)
 {
   int ijk[3];
   ijk[0] = i;
@@ -873,7 +862,7 @@ void vtkUniformGrid::UnBlankPoint(vtkIdType ptId)
 }
 
 //------------------------------------------------------------------------------
-void vtkUniformGrid::UnBlankPoint(const int i, const int j, const int k)
+void vtkUniformGrid::UnBlankPoint(int i, int j, int k)
 {
   int ijk[3];
   ijk[0] = i;
@@ -898,7 +887,7 @@ void vtkUniformGrid::BlankCell(vtkIdType cellId)
 }
 
 //------------------------------------------------------------------------------
-void vtkUniformGrid::BlankCell(const int i, const int j, const int k)
+void vtkUniformGrid::BlankCell(int i, int j, int k)
 {
   int ijk[3];
   ijk[0] = i;
@@ -923,7 +912,7 @@ void vtkUniformGrid::UnBlankCell(vtkIdType cellId)
 }
 
 //------------------------------------------------------------------------------
-void vtkUniformGrid::UnBlankCell(const int i, const int j, const int k)
+void vtkUniformGrid::UnBlankCell(int i, int j, int k)
 {
   int ijk[3];
   ijk[0] = i;
@@ -1083,3 +1072,4 @@ bool vtkUniformGrid::HasAnyBlankCells()
   int cellBlanking = this->CellData->HasAnyGhostBitSet(vtkDataSetAttributes::HIDDENCELL);
   return cellBlanking || this->HasAnyBlankPoints();
 }
+VTK_ABI_NAMESPACE_END
