@@ -1,21 +1,9 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkEnSightReader.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkEnSightReader.h"
 
 #include "vtkDataArrayCollection.h"
-#include "vtkFloatArray.h"
+#include "vtkDoubleArray.h"
 #include "vtkIdList.h"
 #include "vtkIdListCollection.h"
 #include "vtkInformation.h"
@@ -36,6 +24,7 @@
 
 //------------------------------------------------------------------------------
 typedef std::vector<vtkSmartPointer<vtkIdList>> vtkEnSightReaderCellIdsTypeBase;
+VTK_ABI_NAMESPACE_BEGIN
 class vtkEnSightReaderCellIdsType : public vtkEnSightReaderCellIdsTypeBase
 {
 };
@@ -205,7 +194,7 @@ int vtkEnSightReader::RequestData(vtkInformation* vtkNotUsed(request),
   int i, timeSet, fileSet, timeStep, timeStepInFile, fileNum;
   vtkDataArray* times;
   vtkIdList *numStepsList, *filenameNumbers;
-  float newTime;
+  double newTime;
   int numSteps;
   char* fileName;
   int filenameNum;
@@ -440,6 +429,24 @@ int vtkEnSightReader::RequestInformation(vtkInformation* vtkNotUsed(request),
   return this->CaseFileRead;
 }
 
+//-----------------------------------------------------------------------------
+int vtkEnSightReader::ReadCaseFileScripts(char* line)
+{
+  /* The scripts variable is a new area of the EnSight Gold Format which is used
+  to reference an eventual metadata xml file for handling units. The function
+  just skips it.*/
+  int lineRead = this->ReadNextDataLine(line);
+  while (lineRead)
+  {
+    if (strncmp(line, "metadata:", 9) == 0)
+    {
+      vtkDebugMacro("Skipping metadata");
+    }
+    lineRead = this->ReadNextDataLine(line);
+  }
+  return lineRead;
+}
+
 //------------------------------------------------------------------------------
 int vtkEnSightReader::ReadCaseFileGeometry(char* line)
 {
@@ -455,20 +462,20 @@ int vtkEnSightReader::ReadCaseFileGeometry(char* line)
   {
     if (strncmp(line, "model:", 6) == 0)
     {
-      if (sscanf(line, " %*s %d%*[ \t]%d%*[ \t]%s", &timeSet, &fileSet, subLine) == 3)
+      if (sscanf(line, " %*s %d%*[ \t]%d%*[ \t]%[^\t\r\n]", &timeSet, &fileSet, subLine) == 3)
       {
         this->GeometryTimeSet = timeSet;
         this->GeometryFileSet = fileSet;
         this->SetGeometryFileName(subLine);
         vtkDebugMacro(<< this->GetGeometryFileName());
       }
-      else if (sscanf(line, " %*s %d%*[ \t]%s", &timeSet, subLine) == 2)
+      else if (sscanf(line, " %*s %d%*[ \t]%[^\t\r\n]", &timeSet, subLine) == 2)
       {
         this->GeometryTimeSet = timeSet;
         this->SetGeometryFileName(subLine);
         vtkDebugMacro(<< this->GetGeometryFileName());
       }
-      else if (sscanf(line, " %*s %s", subLine) == 1)
+      else if (sscanf(line, " %*s %[^\t\r\n]", subLine) == 1)
       {
         this->SetGeometryFileName(subLine);
         vtkDebugMacro(<< this->GetGeometryFileName());
@@ -476,20 +483,20 @@ int vtkEnSightReader::ReadCaseFileGeometry(char* line)
     }
     else if (strncmp(line, "measured:", 9) == 0)
     {
-      if (sscanf(line, " %*s %d%*[ \t]%d%*[ \t]%s", &timeSet, &fileSet, subLine) == 3)
+      if (sscanf(line, " %*s %d%*[ \t]%d%*[ \t]%[^\t\r\n]", &timeSet, &fileSet, subLine) == 3)
       {
         this->MeasuredTimeSet = timeSet;
         this->MeasuredFileSet = fileSet;
         this->SetMeasuredFileName(subLine);
         vtkDebugMacro(<< this->GetMeasuredFileName());
       }
-      else if (sscanf(line, " %*s %d%*[ \t]%s", &timeSet, subLine) == 2)
+      else if (sscanf(line, " %*s %d%*[ \t]%[^\t\r\n]", &timeSet, subLine) == 2)
       {
         this->MeasuredTimeSet = timeSet;
         this->SetMeasuredFileName(subLine);
         vtkDebugMacro(<< this->GetMeasuredFileName());
       }
-      else if (sscanf(line, " %*s %s", subLine) == 1)
+      else if (sscanf(line, " %*s %[^\t\r\n]", subLine) == 1)
       {
         this->SetMeasuredFileName(subLine);
         vtkDebugMacro(<< this->GetMeasuredFileName());
@@ -497,7 +504,7 @@ int vtkEnSightReader::ReadCaseFileGeometry(char* line)
     }
     else if (strncmp(line, "match:", 6) == 0)
     {
-      sscanf(line, " %*s %s", subLine);
+      sscanf(line, " %*s %[^\t\r\n]", subLine);
       this->SetMatchFileName(subLine);
       vtkDebugMacro(<< this->GetMatchFileName());
     }
@@ -532,7 +539,7 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
   lineRead = this->ReadNextDataLine(line);
   while (lineRead && strncmp(line, "FORMAT", 6) != 0 && strncmp(line, "GEOMETRY", 8) != 0 &&
     strncmp(line, "VARIABLE", 8) != 0 && strncmp(line, "TIME", 4) != 0 &&
-    strncmp(line, "FILE", 4) != 0)
+    strncmp(line, "FILE", 4) != 0 && strncmp(line, "SCRIPTS", 7) != 0)
   {
     if (strncmp(line, "constant", 8) == 0)
     {
@@ -550,19 +557,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         this->NumberOfScalarsPerNode++;
@@ -576,19 +583,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         this->NumberOfScalarsPerElement++;
@@ -602,19 +609,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         this->NumberOfScalarsPerMeasuredNode++;
@@ -634,19 +641,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         this->NumberOfVectorsPerNode++;
@@ -660,19 +667,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         this->NumberOfVectorsPerElement++;
@@ -686,19 +693,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         this->NumberOfVectorsPerMeasuredNode++;
@@ -751,19 +758,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         if (asym)
@@ -785,19 +792,19 @@ int vtkEnSightReader::ReadCaseFileVariable(char* line)
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->VariableFileSetIds->InsertNextId(fileSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %d %s", &timeSet, subLine) == 2)
         {
           this->VariableTimeSetIds->InsertNextId(timeSet);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*d %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*d %*s %[^\t\r\n]", subLine);
         }
         else if (sscanf(line, " %*s %*s %*s %*s %s", subLine) == 1)
         {
           this->VariableTimeSetIds->InsertNextId(1);
           this->AddVariableDescription(subLine);
-          sscanf(line, " %*s %*s %*s %*s %*s %s", subLine);
+          sscanf(line, " %*s %*s %*s %*s %*s %[^\t\r\n]", subLine);
         }
         this->AddVariableType();
         if (asym)
@@ -955,7 +962,7 @@ int vtkEnSightReader::ReadCaseFileTime(char* line)
   char formatLine[256];
   char subLine[256];
   int timeSet, numTimeSteps, i, filenameNum, increment, lineRead;
-  float timeStep;
+  double timeStep;
 
   // found TIME section
   int firstTimeStep = 1;
@@ -1118,14 +1125,14 @@ int vtkEnSightReader::ReadCaseFileTime(char* line)
     }
 
     // 'time values:' --- to obtain timeStep(s)
-    vtkFloatArray* timeValues = vtkFloatArray::New();
+    vtkDoubleArray* timeValues = vtkDoubleArray::New();
     timeValues->SetNumberOfComponents(1);
     timeValues->SetNumberOfTuples(numTimeSteps);
 
     // Time values may be provided on the line(s) following  'time values:',
     // as is usually the case --- not "inline". Thus we need to go to the
     // FIRST line that indeed contains time values.
-    if (sscanf(line, "%*s %*s %f", &timeStep) != 1)
+    if (sscanf(line, "%*s %*s %lf", &timeStep) != 1)
     {
       // not "inline"
       if (this->ReadNextDataLine(line) == 0)
@@ -1146,7 +1153,7 @@ int vtkEnSightReader::ReadCaseFileTime(char* line)
 
     for (i = 0; i < numTimeSteps; i++)
     {
-      strcat(formatLine, "%f ");
+      strcat(formatLine, "%lf ");
 
       // More lines might be needed to provide the remaining time values
       // and then formatLine and subLine need to be updated. 'while' is used
@@ -1167,12 +1174,12 @@ int vtkEnSightReader::ReadCaseFileTime(char* line)
         }
 
         // to access a new line
-        strcpy(formatLine, "%f ");
+        strcpy(formatLine, "%lf ");
         strcpy(subLine, "");
       }
 
       timeValues->SetComponent(i, 0, timeStep);
-      strcat(subLine, "%*f ");
+      strcat(subLine, "%*lf ");
       strcpy(formatLine, subLine);
 
       // init min and max only upon the access to the FIRST 'timeStep'
@@ -1412,6 +1419,12 @@ int vtkEnSightReader::ReadCaseFile()
       vtkDebugMacro(<< "*** FILE section");
       ret = this->ReadCaseFileFile(line);
     }
+    else if (strncmp(line, "SCRIPTS", 7) == 0)
+    {
+      // found SCRIPTS section
+      vtkDebugMacro(<< "*** SCRIPTS section");
+      ret = this->ReadCaseFileScripts(line);
+    }
   }
 
   delete this->IS;
@@ -1431,7 +1444,7 @@ int vtkEnSightReader::ReadVariableFiles(vtkMultiBlockDataSet* output)
   char description[256];
   int timeSet, fileSet, timeStep, timeStepInFile, numSteps;
   vtkDataArray* times;
-  float newTime;
+  double newTime;
   vtkIdList *numStepsList, *filenameNumbers;
   int fileNum;
   int validTime, filenameNum;
@@ -2168,3 +2181,4 @@ void vtkEnSightReader::PrintSelf(ostream& os, vtkIndent indent)
   os << indent << "UseTimeSets: " << this->UseTimeSets << endl;
   os << indent << "UseFileSets: " << this->UseFileSets << endl;
 }
+VTK_ABI_NAMESPACE_END
