@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   Visualization Toolkit
-  Module:    vtkOSPRayPointGaussianMapperNode.cxx
-
-  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
-  All rights reserved.
-  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkOSPRayPointGaussianMapperNode.h"
 
 #include "vtkActor.h"
@@ -50,6 +38,7 @@
 
 namespace vtkosp
 {
+VTK_ABI_NAMESPACE_BEGIN
 //----------------------------------------------------------------------------
 double GetComponent(double* tuple, int nComp, int c)
 {
@@ -142,7 +131,7 @@ void BuildLookupTable(vtkScalarsToColors* cf, float* table, int size, double sca
 
 //----------------------------------------------------------------------------
 float GetScaledRadius(double radius, float* scaleTable, int scaleTableSize, double scaleScale,
-  double scaleOffset, double scaleFactor, double triangleScale)
+  double scaleOffset, double scaleFactor, double boundScale)
 {
   if (scaleTable)
   {
@@ -164,7 +153,7 @@ float GetScaledRadius(double radius, float* scaleTable, int scaleTableSize, doub
   }
 
   radius *= scaleFactor;
-  radius *= triangleScale;
+  radius *= boundScale;
   if (radius < 1e-3)
   {
     radius *= 1e2;
@@ -177,7 +166,7 @@ float GetScaledRadius(double radius, float* scaleTable, int scaleTableSize, doub
 
 //----------------------------------------------------------------------------
 OSPVolumetricModel RenderAsParticles(osp::vec3f* vertices, std::vector<unsigned int>& indexArray,
-  double pointSize, double scaleFactor, double triangleScale, vtkDataArray* scaleArray,
+  double pointSize, double scaleFactor, double boundScale, vtkDataArray* scaleArray,
   int scaleArrayComponent, float* scaleTable, int scaleTableSize, double scaleScale,
   double scaleOffset, vtkDataArray* vtkNotUsed(opacityArray), float* vtkNotUsed(opacityTable),
   int vtkNotUsed(opacityTableSize), double vtkNotUsed(opacityScale),
@@ -220,7 +209,7 @@ OSPVolumetricModel RenderAsParticles(osp::vec3f* vertices, std::vector<unsigned
         scaleArray->GetNumberOfComponents(), scaleArrayComponent);
     }
     float r = vtkosp::GetScaledRadius(
-      rDouble, scaleTable, scaleTableSize, scaleScale, scaleOffset, scaleFactor, triangleScale);
+      rDouble, scaleTable, scaleTableSize, scaleScale, scaleOffset, scaleFactor, boundScale);
     radii.emplace_back(r);
 
     float weight = 1.f;
@@ -306,8 +295,11 @@ OSPVolumetricModel RenderAsParticles(osp::vec3f* vertices, std::vector<unsigned
 
   return ospVolModel;
 }
+VTK_ABI_NAMESPACE_END
 }
 
+VTK_ABI_NAMESPACE_BEGIN
+
 //============================================================================
 vtkStandardNewMacro(vtkOSPRayPointGaussianMapperNode);
 
@@ -338,7 +330,8 @@ void vtkOSPRayPointGaussianMapperNode::PrintSelf(ostream& os, vtkIndent indent)
 
 //------------------------------------------------------------------------------
 void vtkOSPRayPointGaussianMapperNode::InternalRender(void* vtkNotUsed(renderer),
-  vtkOSPRayActorNode* aNode, vtkPolyData* poly, double opacity, std::string materialName)
+  vtkOSPRayActorNode* aNode, vtkPolyData* poly, double vtkNotUsed(opacity),
+  std::string vtkNotUsed(materialName))
 {
   vtkOSPRayRendererNode* orn =
     static_cast<vtkOSPRayRendererNode*>(this->GetFirstAncestorOfType("vtkOSPRayRendererNode"));
@@ -466,7 +459,7 @@ void vtkOSPRayPointGaussianMapperNode::InternalRender(void* vtkNotUsed(renderer)
   }
 
   this->VolumetricModels.emplace_back(vtkosp::RenderAsParticles(vertices.data(), conn.vertex_index,
-    pointSize, mapper->GetScaleFactor(), mapper->GetTriangleScale(), scaleArray,
+    pointSize, mapper->GetScaleFactor(), mapper->GetBoundScale(), scaleArray,
     mapper->GetScaleArrayComponent(), this->ScaleTable, this->ScaleTableSize, this->ScaleScale,
     this->ScaleOffset, opacityArray, this->OpacityTable, this->OpacityTableSize, this->OpacityScale,
     this->OpacityOffset, scalarArray, lut, this->NumColors, backend));
@@ -600,3 +593,4 @@ bool vtkOSPRayPointGaussianMapperNode::GetNeedToRebuild(vtkOSPRayActorNode* aNod
   }
   return false;
 }
+VTK_ABI_NAMESPACE_END
