@@ -1,17 +1,5 @@
-/*=========================================================================
-
-  Program:   ParaView
-  Module:    vtkSMExtractsController.cxx
-
-  Copyright (c) Kitware, Inc.
-  All rights reserved.
-  See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-     PURPOSE.  See the above copyright notice for more information.
-
-=========================================================================*/
+// SPDX-FileCopyrightText: Copyright (c) Kitware Inc.
+// SPDX-License-Identifier: BSD-3-Clause
 #include "vtkSMExtractsController.h"
 
 #include "vtkCollection.h"
@@ -480,7 +468,10 @@ bool vtkSMExtractsController::CreateDir(
   }
   else
   {
-    vtkErrorMacro("Failed to create directory: " << dname.c_str());
+    // Expand directory name for better error message
+    std::string fullPath = vtksys::SystemTools::CollapseFullPath(dname);
+
+    vtkErrorMacro("Failed to create directory: " << fullPath.c_str());
     return false;
   }
 }
